
package org.mule.modules.avalara.processors;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.WildcardType;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;
import org.mule.api.MessagingException;
import org.mule.api.MuleContext;
import org.mule.api.MuleEvent;
import org.mule.api.MuleException;
import org.mule.api.MuleMessage;
import org.mule.api.construct.FlowConstruct;
import org.mule.api.construct.FlowConstructAware;
import org.mule.api.context.MuleContextAware;
import org.mule.api.expression.ExpressionManager;
import org.mule.api.lifecycle.Disposable;
import org.mule.api.lifecycle.Initialisable;
import org.mule.api.lifecycle.InitialisationException;
import org.mule.api.lifecycle.Startable;
import org.mule.api.lifecycle.Stoppable;
import org.mule.api.processor.MessageProcessor;
import org.mule.api.registry.RegistrationException;
import org.mule.api.transformer.DataType;
import org.mule.api.transformer.Transformer;
import org.mule.api.transformer.TransformerException;
import org.mule.config.i18n.CoreMessages;
import org.mule.config.i18n.MessageFactory;
import org.mule.modules.avalara.AvalaraModule;
import org.mule.modules.avalara.adapters.AvalaraModuleLifecycleAdapter;
import org.mule.transformer.TransformerTemplate;
import org.mule.transformer.types.DataTypeFactory;
import org.mule.transport.NullPayload;
import org.mule.util.TemplateParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * GetTaxMessageProcessor invokes the {@link org.mule.modules.avalara.AvalaraModule#getTax(java.lang.String, java.lang.String, java.lang.String, java.lang.String, org.mule.modules.avalara.AvalaraDocumentType, java.lang.String, java.util.Date, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.util.List, java.util.List, org.mule.modules.avalara.DetailLevelType, java.lang.String, java.lang.String, boolean, java.lang.String, java.util.Map, java.lang.String, org.mule.modules.avalara.ServiceModeType, java.util.Date, java.lang.String, java.util.Date)} method in {@link AvalaraModule }. For each argument there is a field in this processor to match it.  Before invoking the actual method the processor will evaluate and transform where possible to the expected argument type.
 * 
 */
public class GetTaxMessageProcessor
    implements FlowConstructAware, MuleContextAware, Disposable, Initialisable, Startable, Stoppable, MessageProcessor
{

    private Object account;
    private String _accountType;
    private Object license;
    private String _licenseType;
    private Object avalaraClient;
    private String _avalaraClientType;
    private Object companyCode;
    private String _companyCodeType;
    private Object docType;
    private org.mule.modules.avalara.AvalaraDocumentType _docTypeType;
    private Object docCode;
    private String _docCodeType;
    private Object docDate;
    private Date _docDateType;
    private Object salespersonCode;
    private String _salespersonCodeType;
    private Object customerCode;
    private String _customerCodeType;
    private Object customerUsageType;
    private String _customerUsageTypeType;
    private Object discount;
    private String _discountType;
    private Object purchaseOrderNo;
    private String _purchaseOrderNoType;
    private Object exemptionNo;
    private String _exemptionNoType;
    private Object originCode;
    private String _originCodeType;
    private Object destinationCode;
    private String _destinationCodeType;
    private Object baseAddresses;
    private List<Map<String, Object>> _baseAddressesType;
    private Object lines;
    private List<Map<String, Object>> _linesType;
    private Object detailLevel;
    private org.mule.modules.avalara.DetailLevelType _detailLevelType;
    private Object referenceCode;
    private String _referenceCodeType;
    private Object locationCode;
    private String _locationCodeType;
    private Object commit;
    private boolean _commitType;
    private Object batchCode;
    private String _batchCodeType;
    private Object taxOverride;
    private Map<String, Object> _taxOverrideType;
    private Object currencyCode;
    private String _currencyCodeType;
    private Object serviceMode;
    private org.mule.modules.avalara.ServiceModeType _serviceModeType;
    private Object paymentDate;
    private Date _paymentDateType;
    private Object exchangeRate;
    private String _exchangeRateType;
    private Object exchangeRateEffDate;
    private Date _exchangeRateEffDateType;
    private static Logger logger = LoggerFactory.getLogger(GetTaxMessageProcessor.class);
    /**
     * Module object
     * 
     */
    private Object moduleObject;
    /**
     * Mule Context
     * 
     */
    private MuleContext muleContext;
    /**
     * Mule Expression Manager
     * 
     */
    private ExpressionManager expressionManager;
    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;
    /**
     * Flow construct
     * 
     */
    private FlowConstruct flowConstruct;
    /**
     * Variable used to track how many retries we have attempted on this message processor
     * 
     */
    private AtomicInteger retryCount;
    /**
     * Maximum number of retries that can be attempted.
     * 
     */
    private int retryMax;

    /**
     * Obtains the expression manager from the Mule context and initialises the connector. If a target object  has not been set already it will search the Mule registry for a default one.
     * 
     * @throws InitialisationException
     */
    public void initialise()
        throws InitialisationException
    {
        retryCount = new AtomicInteger();
        expressionManager = muleContext.getExpressionManager();
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
        if (moduleObject == null) {
            try {
                moduleObject = muleContext.getRegistry().lookupObject(AvalaraModuleLifecycleAdapter.class);
                if (moduleObject == null) {
                    moduleObject = new AvalaraModuleLifecycleAdapter();
                    muleContext.getRegistry().registerObject(AvalaraModuleLifecycleAdapter.class.getName(), moduleObject);
                }
            } catch (RegistrationException e) {
                throw new InitialisationException(CoreMessages.initialisationFailure("org.mule.modules.avalara.adapters.AvalaraModuleLifecycleAdapter"), e, this);
            }
        }
        if (moduleObject instanceof String) {
            moduleObject = muleContext.getRegistry().lookupObject(((String) moduleObject));
            if (moduleObject == null) {
                throw new InitialisationException(MessageFactory.createStaticMessage("Cannot find object by config name"), this);
            }
        }
    }

    public void start()
        throws MuleException
    {
    }

    public void stop()
        throws MuleException
    {
    }

    public void dispose() {
    }

    /**
     * Set the Mule context
     * 
     * @param context Mule context to set
     */
    public void setMuleContext(MuleContext context) {
        this.muleContext = context;
    }

    /**
     * Sets flow construct
     * 
     * @param flowConstruct Flow construct to set
     */
    public void setFlowConstruct(FlowConstruct flowConstruct) {
        this.flowConstruct = flowConstruct;
    }

    /**
     * Sets the instance of the object under which the processor will execute
     * 
     * @param moduleObject Instace of the module
     */
    public void setModuleObject(Object moduleObject) {
        this.moduleObject = moduleObject;
    }

    /**
     * Sets retryMax
     * 
     * @param value Value to set
     */
    public void setRetryMax(int value) {
        this.retryMax = value;
    }

    /**
     * Sets customerUsageType
     * 
     * @param value Value to set
     */
    public void setCustomerUsageType(Object value) {
        this.customerUsageType = value;
    }

    /**
     * Sets docType
     * 
     * @param value Value to set
     */
    public void setDocType(Object value) {
        this.docType = value;
    }

    /**
     * Sets referenceCode
     * 
     * @param value Value to set
     */
    public void setReferenceCode(Object value) {
        this.referenceCode = value;
    }

    /**
     * Sets batchCode
     * 
     * @param value Value to set
     */
    public void setBatchCode(Object value) {
        this.batchCode = value;
    }

    /**
     * Sets taxOverride
     * 
     * @param value Value to set
     */
    public void setTaxOverride(Object value) {
        this.taxOverride = value;
    }

    /**
     * Sets exchangeRate
     * 
     * @param value Value to set
     */
    public void setExchangeRate(Object value) {
        this.exchangeRate = value;
    }

    /**
     * Sets commit
     * 
     * @param value Value to set
     */
    public void setCommit(Object value) {
        this.commit = value;
    }

    /**
     * Sets customerCode
     * 
     * @param value Value to set
     */
    public void setCustomerCode(Object value) {
        this.customerCode = value;
    }

    /**
     * Sets docDate
     * 
     * @param value Value to set
     */
    public void setDocDate(Object value) {
        this.docDate = value;
    }

    /**
     * Sets license
     * 
     * @param value Value to set
     */
    public void setLicense(Object value) {
        this.license = value;
    }

    /**
     * Sets baseAddresses
     * 
     * @param value Value to set
     */
    public void setBaseAddresses(Object value) {
        this.baseAddresses = value;
    }

    /**
     * Sets paymentDate
     * 
     * @param value Value to set
     */
    public void setPaymentDate(Object value) {
        this.paymentDate = value;
    }

    /**
     * Sets currencyCode
     * 
     * @param value Value to set
     */
    public void setCurrencyCode(Object value) {
        this.currencyCode = value;
    }

    /**
     * Sets serviceMode
     * 
     * @param value Value to set
     */
    public void setServiceMode(Object value) {
        this.serviceMode = value;
    }

    /**
     * Sets avalaraClient
     * 
     * @param value Value to set
     */
    public void setAvalaraClient(Object value) {
        this.avalaraClient = value;
    }

    /**
     * Sets detailLevel
     * 
     * @param value Value to set
     */
    public void setDetailLevel(Object value) {
        this.detailLevel = value;
    }

    /**
     * Sets exemptionNo
     * 
     * @param value Value to set
     */
    public void setExemptionNo(Object value) {
        this.exemptionNo = value;
    }

    /**
     * Sets originCode
     * 
     * @param value Value to set
     */
    public void setOriginCode(Object value) {
        this.originCode = value;
    }

    /**
     * Sets exchangeRateEffDate
     * 
     * @param value Value to set
     */
    public void setExchangeRateEffDate(Object value) {
        this.exchangeRateEffDate = value;
    }

    /**
     * Sets purchaseOrderNo
     * 
     * @param value Value to set
     */
    public void setPurchaseOrderNo(Object value) {
        this.purchaseOrderNo = value;
    }

    /**
     * Sets discount
     * 
     * @param value Value to set
     */
    public void setDiscount(Object value) {
        this.discount = value;
    }

    /**
     * Sets companyCode
     * 
     * @param value Value to set
     */
    public void setCompanyCode(Object value) {
        this.companyCode = value;
    }

    /**
     * Sets destinationCode
     * 
     * @param value Value to set
     */
    public void setDestinationCode(Object value) {
        this.destinationCode = value;
    }

    /**
     * Sets locationCode
     * 
     * @param value Value to set
     */
    public void setLocationCode(Object value) {
        this.locationCode = value;
    }

    /**
     * Sets salespersonCode
     * 
     * @param value Value to set
     */
    public void setSalespersonCode(Object value) {
        this.salespersonCode = value;
    }

    /**
     * Sets account
     * 
     * @param value Value to set
     */
    public void setAccount(Object value) {
        this.account = value;
    }

    /**
     * Sets lines
     * 
     * @param value Value to set
     */
    public void setLines(Object value) {
        this.lines = value;
    }

    /**
     * Sets docCode
     * 
     * @param value Value to set
     */
    public void setDocCode(Object value) {
        this.docCode = value;
    }

    /**
     * Get all superclasses and interfaces recursively.
     * 
     * @param classes List of classes to which to add all found super classes and interfaces.
     * @param clazz   The class to start the search with.
     */
    private void computeClassHierarchy(Class clazz, List classes) {
        for (Class current = clazz; (current!= null); current = current.getSuperclass()) {
            if (classes.contains(current)) {
                return ;
            }
            classes.add(current);
            for (Class currentInterface: current.getInterfaces()) {
                computeClassHierarchy(currentInterface, classes);
            }
        }
    }

    /**
     * Checks whether the specified class parameter is an instance of {@link List }
     * 
     * @param clazz <code>Class</code> to check.
     * @return
     */
    private boolean isListClass(Class clazz) {
        List<Class> classes = new ArrayList<Class>();
        computeClassHierarchy(clazz, classes);
        return classes.contains(List.class);
    }

    /**
     * Checks whether the specified class parameter is an instance of {@link Map }
     * 
     * @param clazz <code>Class</code> to check.
     * @return
     */
    private boolean isMapClass(Class clazz) {
        List<Class> classes = new ArrayList<Class>();
        computeClassHierarchy(clazz, classes);
        return classes.contains(Map.class);
    }

    private boolean isList(Type type) {
        if ((type instanceof Class)&&isListClass(((Class) type))) {
            return true;
        }
        if (type instanceof ParameterizedType) {
            return isList(((ParameterizedType) type).getRawType());
        }
        if (type instanceof WildcardType) {
            Type[] upperBounds = ((WildcardType) type).getUpperBounds();
            return ((upperBounds.length!= 0)&&isList(upperBounds[ 0 ]));
        }
        return false;
    }

    private boolean isMap(Type type) {
        if ((type instanceof Class)&&isMapClass(((Class) type))) {
            return true;
        }
        if (type instanceof ParameterizedType) {
            return isMap(((ParameterizedType) type).getRawType());
        }
        if (type instanceof WildcardType) {
            Type[] upperBounds = ((WildcardType) type).getUpperBounds();
            return ((upperBounds.length!= 0)&&isMap(upperBounds[ 0 ]));
        }
        return false;
    }

    private boolean isAssignableFrom(Type expectedType, Class clazz) {
        if (expectedType instanceof Class) {
            if (((Class) expectedType).isPrimitive()) {
                if (((Class) expectedType).getName().equals("boolean")&&(clazz == Boolean.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("byte")&&(clazz == Byte.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("short")&&(clazz == Short.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("char")&&(clazz == Character.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("int")&&(clazz == Integer.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("float")&&(clazz == Float.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("long")&&(clazz == Long.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("double")&&(clazz == Double.class)) {
                    return true;
                }
                return false;
            } else {
                return ((Class) expectedType).isAssignableFrom(clazz);
            }
        }
        if (expectedType instanceof ParameterizedType) {
            return isAssignableFrom(((ParameterizedType) expectedType).getRawType(), clazz);
        }
        if (expectedType instanceof WildcardType) {
            Type[] upperBounds = ((WildcardType) expectedType).getUpperBounds();
            if (upperBounds.length!= 0) {
                return isAssignableFrom(upperBounds[ 0 ], clazz);
            }
        }
        return false;
    }

    private Object evaluate(MuleMessage muleMessage, Object source) {
        if (source instanceof String) {
            String stringSource = ((String) source);
            if (stringSource.startsWith(patternInfo.getPrefix())&&stringSource.endsWith(patternInfo.getSuffix())) {
                return expressionManager.evaluate(stringSource, muleMessage);
            } else {
                return expressionManager.parse(stringSource, muleMessage);
            }
        }
        return source;
    }

    private Object evaluateAndTransform(MuleMessage muleMessage, Type expectedType, String expectedMimeType, Object source)
        throws TransformerException
    {
        if (source == null) {
            return source;
        }
        Object target = null;
        if (isList(source.getClass())) {
            if (isList(expectedType)) {
                List newList = new ArrayList();
                Type valueType = ((ParameterizedType) expectedType).getActualTypeArguments()[ 0 ];
                ListIterator iterator = ((List) source).listIterator();
                while (iterator.hasNext()) {
                    Object subTarget = iterator.next();
                    newList.add(evaluateAndTransform(muleMessage, valueType, expectedMimeType, subTarget));
                }
                target = newList;
            } else {
                target = source;
            }
        } else {
            if (isMap(source.getClass())) {
                if (isMap(expectedType)) {
                    Type keyType = Object.class;
                    Type valueType = Object.class;
                    if (expectedType instanceof ParameterizedType) {
                        keyType = ((ParameterizedType) expectedType).getActualTypeArguments()[ 0 ];
                        valueType = ((ParameterizedType) expectedType).getActualTypeArguments()[ 1 ];
                    }
                    Map map = ((Map) source);
                    Map newMap = new HashMap();
                    for (Object entryObj: map.entrySet()) {
                        {
                            Map.Entry entry = ((Map.Entry) entryObj);
                            Object newKey = evaluateAndTransform(muleMessage, keyType, expectedMimeType, entry.getKey());
                            Object newValue = evaluateAndTransform(muleMessage, valueType, expectedMimeType, entry.getValue());
                            newMap.put(newKey, newValue);
                        }
                    }
                    target = newMap;
                } else {
                    target = source;
                }
            } else {
                target = evaluate(muleMessage, source);
            }
        }
        if ((target!= null)&&(!isAssignableFrom(expectedType, target.getClass()))) {
            DataType sourceDataType = DataTypeFactory.create(target.getClass());
            DataType targetDataType = null;
            if (expectedMimeType!= null) {
                targetDataType = DataTypeFactory.create(((Class) expectedType), expectedMimeType);
            } else {
                targetDataType = DataTypeFactory.create(((Class) expectedType));
            }
            Transformer t = muleContext.getRegistry().lookupTransformer(sourceDataType, targetDataType);
            return t.transform(target);
        } else {
            return target;
        }
    }

    /**
     * Invokes the MessageProcessor.
     * 
     * @param event MuleEvent to be processed
     * @throws MuleException
     */
    public MuleEvent process(MuleEvent event)
        throws MuleException
    {
        MuleMessage _muleMessage = event.getMessage();
        AvalaraModuleLifecycleAdapter _castedModuleObject = null;
        if (moduleObject instanceof String) {
            _castedModuleObject = ((AvalaraModuleLifecycleAdapter) muleContext.getRegistry().lookupObject(((String) moduleObject)));
            if (_castedModuleObject == null) {
                throw new MessagingException(CoreMessages.failedToCreate("getTax"), event, new RuntimeException("Cannot find the configuration specified by the config-ref attribute."));
            }
        } else {
            _castedModuleObject = ((AvalaraModuleLifecycleAdapter) moduleObject);
        }
        try {
            String _transformedAccount = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_accountType").getGenericType(), null, account));
            String _transformedLicense = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_licenseType").getGenericType(), null, license));
            String _transformedAvalaraClient = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_avalaraClientType").getGenericType(), null, avalaraClient));
            String _transformedCompanyCode = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_companyCodeType").getGenericType(), null, companyCode));
            org.mule.modules.avalara.AvalaraDocumentType _transformedDocType = ((org.mule.modules.avalara.AvalaraDocumentType) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_docTypeType").getGenericType(), null, docType));
            String _transformedDocCode = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_docCodeType").getGenericType(), null, docCode));
            Date _transformedDocDate = ((Date) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_docDateType").getGenericType(), null, docDate));
            String _transformedSalespersonCode = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_salespersonCodeType").getGenericType(), null, salespersonCode));
            String _transformedCustomerCode = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_customerCodeType").getGenericType(), null, customerCode));
            String _transformedCustomerUsageType = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_customerUsageTypeType").getGenericType(), null, customerUsageType));
            String _transformedDiscount = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_discountType").getGenericType(), null, discount));
            String _transformedPurchaseOrderNo = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_purchaseOrderNoType").getGenericType(), null, purchaseOrderNo));
            String _transformedExemptionNo = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_exemptionNoType").getGenericType(), null, exemptionNo));
            String _transformedOriginCode = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_originCodeType").getGenericType(), null, originCode));
            String _transformedDestinationCode = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_destinationCodeType").getGenericType(), null, destinationCode));
            List<Map<String, Object>> _transformedBaseAddresses = ((List<Map<String, Object>> ) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_baseAddressesType").getGenericType(), null, baseAddresses));
            List<Map<String, Object>> _transformedLines = ((List<Map<String, Object>> ) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_linesType").getGenericType(), null, lines));
            org.mule.modules.avalara.DetailLevelType _transformedDetailLevel = ((org.mule.modules.avalara.DetailLevelType) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_detailLevelType").getGenericType(), null, detailLevel));
            String _transformedReferenceCode = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_referenceCodeType").getGenericType(), null, referenceCode));
            String _transformedLocationCode = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_locationCodeType").getGenericType(), null, locationCode));
            Boolean _transformedCommit = ((Boolean) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_commitType").getGenericType(), null, commit));
            String _transformedBatchCode = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_batchCodeType").getGenericType(), null, batchCode));
            Map<String, Object> _transformedTaxOverride = ((Map<String, Object> ) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_taxOverrideType").getGenericType(), null, taxOverride));
            String _transformedCurrencyCode = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_currencyCodeType").getGenericType(), null, currencyCode));
            org.mule.modules.avalara.ServiceModeType _transformedServiceMode = ((org.mule.modules.avalara.ServiceModeType) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_serviceModeType").getGenericType(), null, serviceMode));
            Date _transformedPaymentDate = ((Date) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_paymentDateType").getGenericType(), null, paymentDate));
            String _transformedExchangeRate = ((String) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_exchangeRateType").getGenericType(), null, exchangeRate));
            Date _transformedExchangeRateEffDate = ((Date) evaluateAndTransform(_muleMessage, GetTaxMessageProcessor.class.getDeclaredField("_exchangeRateEffDateType").getGenericType(), null, exchangeRateEffDate));
            retryCount.getAndIncrement();
            Object resultPayload;
            resultPayload = _castedModuleObject.getTax(_transformedAccount, _transformedLicense, _transformedAvalaraClient, _transformedCompanyCode, _transformedDocType, _transformedDocCode, _transformedDocDate, _transformedSalespersonCode, _transformedCustomerCode, _transformedCustomerUsageType, _transformedDiscount, _transformedPurchaseOrderNo, _transformedExemptionNo, _transformedOriginCode, _transformedDestinationCode, _transformedBaseAddresses, _transformedLines, _transformedDetailLevel, _transformedReferenceCode, _transformedLocationCode, _transformedCommit, _transformedBatchCode, _transformedTaxOverride, _transformedCurrencyCode, _transformedServiceMode, _transformedPaymentDate, _transformedExchangeRate, _transformedExchangeRateEffDate);
            TransformerTemplate.OverwitePayloadCallback overwritePayloadCallback = null;
            if (resultPayload == null) {
                overwritePayloadCallback = new TransformerTemplate.OverwitePayloadCallback(NullPayload.getInstance());
            } else {
                overwritePayloadCallback = new TransformerTemplate.OverwitePayloadCallback(resultPayload);
            }
            List<Transformer> transformerList;
            transformerList = new ArrayList<Transformer>();
            transformerList.add(new TransformerTemplate(overwritePayloadCallback));
            event.getMessage().applyTransformers(event, transformerList);
            retryCount.set(0);
            return event;
        } catch (Exception e) {
            throw new MessagingException(CoreMessages.failedToInvoke("getTax"), event, e);
        }
    }

}
